@@ -104,7 +104,8 @@ _gnutls_encrypt (gnutls_session_t session, const uint8_t * headers,
       if (comp.data == NULL)
         return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
       
-      ret = _gnutls_compress( &params->write.compression_state, data, data_size, comp.data, comp.size);
+      ret = _gnutls_compress(&params->write.compression_state, data, data_size, 
+                             comp.data, comp.size, session->internals.priorities.stateless_compression);
       if (ret < 0)
         {
           gnutls_free(comp.data);
@@ -440,10 +441,10 @@ ciphertext_to_compressed (gnutls_session_t session,
                           uint64* sequence)
 {
   uint8_t tag[MAX_HASH_SIZE];
-  uint8_t pad;
+  unsigned int pad, i;
   int length, length_to_decrypt;
   uint16_t blocksize;
-  int ret, i, pad_failed = 0;
+  int ret, pad_failed = 0;
   uint8_t preamble[MAX_PREAMBLE_SIZE];
   unsigned int preamble_size;
   unsigned int ver = gnutls_protocol_get_version (session);
@@ -540,7 +541,7 @@ ciphertext_to_compressed (gnutls_session_t session,
       pad = ciphertext->data[ciphertext->size - 1] + 1;   /* pad */
 
 
-      if ((int) pad > (int) ciphertext->size - tag_size)
+      if (pad > (int) ciphertext->size - tag_size)
         {
           gnutls_assert ();
           _gnutls_record_log
@@ -558,7 +559,7 @@ ciphertext_to_compressed (gnutls_session_t session,
       /* Check the pading bytes (TLS 1.x)
        */
       if (ver != GNUTLS_SSL3)
-        for (i = 2; i < pad; i++)
+        for (i = 2; i <= pad; i++)
           {
             if (ciphertext->data[ciphertext->size - i] !=
                 ciphertext->data[ciphertext->size - 1])
