@@ -28,14 +28,6 @@
 
 #include <libtasn1.h>
 
-/* Remove these when we require libtasn1 v1.6 or later. */
-#ifndef ASN1_MAX_NAME_SIZE
-#define ASN1_MAX_NAME_SIZE MAX_NAME_SIZE
-#endif
-#ifndef ASN1_MAX_ERROR_DESCRIPTION_SIZE
-#define ASN1_MAX_ERROR_DESCRIPTION_SIZE MAX_ERROR_DESCRIPTION_SIZE
-#endif
-
 #define MAX_CRQ_EXTENSIONS_SIZE 8*1024
 #define MAX_OID_SIZE 128
 
@@ -58,6 +50,9 @@ typedef struct gnutls_x509_crt_int
 {
   ASN1_TYPE cert;
   int use_extensions;
+  int expanded; /* a certificate has been expanded */
+  
+  struct pin_info_st pin;
 } gnutls_x509_crt_int;
 
 typedef struct gnutls_x509_crq_int
@@ -116,10 +111,12 @@ int _gnutls_x509_parse_dn (ASN1_TYPE asn1_struct,
                            const char *asn1_rdn_name, char *buf,
                            size_t * sizeof_buf);
 
-int _gnutls_x509_parse_dn_oid (ASN1_TYPE asn1_struct,
-                               const char *asn1_rdn_name, const char *oid,
-                               int indx, unsigned int raw_flag, void *buf,
-                               size_t * sizeof_buf);
+int
+_gnutls_x509_parse_dn_oid (ASN1_TYPE asn1_struct,
+                           const char *asn1_rdn_name,
+                           const char *given_oid, int indx,
+                           unsigned int raw_flag,
+                           gnutls_datum_t* out);
 
 int _gnutls_x509_set_dn_oid (ASN1_TYPE asn1_struct,
                              const char *asn1_rdn_name, const char *oid,
@@ -151,10 +148,6 @@ int _gnutls_x509_verify_data (gnutls_digest_algorithm_t algo,
                               const gnutls_datum_t * signature,
                               gnutls_x509_crt_t issuer);
 
-int _gnutls_x509_verify_hashed_data (const gnutls_datum_t * hash,
-                                     const gnutls_datum_t * signature,
-                                     gnutls_x509_crt_t issuer);
-
 /* privkey.h */
 ASN1_TYPE _gnutls_privkey_decode_pkcs1_rsa_key (const gnutls_datum_t *
                                                 raw_key,
